package irnatura.offline;

//import android.graphics.Color; // HSVToRGB
//import java.awt.Color; // HSBtoRGB


public class HSVPalette
{
    // HSV floats
    public static final float [][] EARTHY = {
        {  60.0f,  50.0f, 100.0f }, // #ffff80 yellow/white
//      {   0.0f,  50.0f, 100.0f }, // #ff8080 bright magenta-ish
        {   0.0f, 100.0f, 100.0f }, // #ff0000 red
        {   0.0f, 100.0f,  50.0f }, // #800000 half-red
//      {   0.0f,   0.0f,   0.0f }, // #000000 black
        { 120.0f, 100.0f,  50.0f }, // #008000 half-green
        { 180.0f, 100.0f,   0.0f }, // #008080 blue-green
        { 210.0f, 100.0f, 100.0f }, // #0080ff half-blue
        { 240.0f, 100.0f,  50.0f }, // #0000ff blue
    };

    private float [][] m_hsv_checkpoints;
    private int [] m_temp_boundaries;

    public HSVPalette(float [][] hsv_checkpoints, int [] temp_boundaries)
    {
        m_hsv_checkpoints = hsv_checkpoints;
        m_temp_boundaries = temp_boundaries;
        // TODO: assert hsv_checkpoints.length >= 2
        // TODO: assert temp_boundaries.length == hsv_checkpoints.length
        // TODO: assert temp_boundaries is sorted
    }

    // Meant for interpolation, but tolerates extrapolation too
    // (with proportion outside of [0.0, 1.0]).
    public static void polate(float proportion,
                              float [] lb,
                              float [] ub,
                              float [] out)
    {
        // Hue: mod 360
        if (Math.abs(ub[0] - lb[0]) < 180.0f) {
            out[0] = (lb[0] + (ub[0] - lb[0]) * proportion) % 360.0f;;
        }
        else {
            float lb0 = lb[0];
            float ub0 = ub[0];
            if (lb0 < ub0) lb0 += 360.0f;
            else ub0 += 360.0f;
            out[0] = (lb0 + (ub0 - lb0) * proportion) % 360.0f;
        }
        out[1] = lb[1] + (ub[1] - lb[1]) * proportion;
        out[2] = lb[2] + (ub[2] - lb[2]) * proportion;
        // paranoid (supports proportion outside of [0.0, 1.0])
        if (out[1] < 0.0f) out[1] = 0.0f;
        if (out[2] < 0.0f) out[2] = 0.0f;
        if (out[1] > 100.0f) out[1] = 100.0f;
        if (out[2] > 100.0f) out[2] = 100.0f;
    }

    // For now: linear search.
    // TODO: binary search.
    public static int find_lteq(int [] sorted_array, int value)
    {
        int i;
        for (i = 0; i < sorted_array.length; ++i) {
            if (sorted_array[i] >= value)
                break;
        }
        return i;
    }

    public void temp2hsv(int temp, float [] out_hsv)
    {
        int lb_index = find_lteq(m_temp_boundaries, temp);
        if (lb_index > m_temp_boundaries.length - 2)
            lb_index = m_temp_boundaries.length - 2;
        int lb = m_temp_boundaries[lb_index];
        int ub = m_temp_boundaries[lb_index + 1];
        // Note: temp should normally be between lb and ub, but we allow
        // for it to go out of bounds.
        float proportion = ((float)(temp - lb)) / ((float)(ub - lb));
        polate(proportion,
               m_hsv_checkpoints[lb_index],
               m_hsv_checkpoints[lb_index + 1],
               out_hsv);
    }
}
